@@ -1,6 +1,6 @@
 /*
  * (C) Copyright 2012
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * Stefano Babic, stefano.babic@swupdate.org.
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
@@ -514,9 +514,9 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 
 	if (encrypted) {
 		aes_key = get_aes_key();
-		if (imgivt && strlen(imgivt)) {
-			if(ascii_to_bin(ivtbuf, sizeof(ivtbuf), imgivt)) {
-				ERROR("invalid image ivt length");
+		if (imgivt) {
+			if (!strlen(imgivt) || !is_hex_str(imgivt) || ascii_to_bin(ivtbuf, sizeof(ivtbuf), imgivt)) {
+				ERROR("Invalid image ivt");
 				return -EINVAL;
 			}
 			ivt = ivtbuf;
@@ -805,52 +805,6 @@ int extract_img_from_cpio(int fd, unsigned long offset, struct filehdr *fdh)
 	return 0;
 }
 
-off_t extract_next_file(int fd, int fdout, off_t start, int compressed,
-		int encrypted, char *ivt, unsigned char *hash)
-{
-	int ret;
-	struct filehdr fdh;
-	uint32_t checksum = 0;
-	unsigned long offset = start;
-
-	ret = lseek(fd, offset, SEEK_SET);
-	if (ret < 0) {
-		ERROR("CPIO file corrupted : %s",
-		strerror(errno));
-		return ret;
-	}
-
-	ret = extract_cpio_header(fd, &fdh, &offset);
-	if (ret) {
-		ERROR("CPIO Header wrong");
-		return ret;
-	}
-
-	ret = lseek(fd, offset, SEEK_SET);
-	if (ret < 0) {
-		ERROR("CPIO file corrupted : %s", strerror(errno));
-		return ret;
-	}
-
-	ret = copyfile(fd, &fdout, fdh.size, &offset, 0, 0, compressed, &checksum, hash, encrypted, ivt, NULL);
-	if (ret < 0) {
-		ERROR("Error copying extracted file");
-		return ret;
-	}
-
-	TRACE("Copied file:\n\tfilename %s\n\tsize %u\n\tchecksum 0x%lx %s",
-		fdh.filename,
-		(unsigned int)fdh.size,
-		(unsigned long)checksum,
-		(checksum == fdh.chksum) ? "VERIFIED" : "WRONG");
-
-	if (!swupdate_verify_chksum(checksum, &fdh)) {
-		return -EINVAL;
-	}
-
-	return offset;
-}
-
 int cpio_scan(int fd, struct swupdate_cfg *cfg, off_t start)
 {
 	struct filehdr fdh;
