@@ -11,6 +11,8 @@
 #include <handler.h>
 #include <blkid/blkid.h>
 #include <fs_interface.h>
+#include "progress.h"
+#include "swupdate_image.h"
 
 void diskformat_handler(void);
 
@@ -51,6 +53,12 @@ static int diskformat(struct img_type *img,
 
 	/* File system does not exist, create new file system */
 	ret = diskformat_mkfs(img->device, fstype);
+
+	/*
+	 * Declare that handler has finished
+	 */
+	swupdate_progress_update(100);
+
 	return ret;
 }
 
