@@ -7,18 +7,14 @@
 
 #pragma once
 #include "util.h"
-#ifdef CONFIG_JSON
 #include <json-c/json.h>
-#endif
 #include <stdio.h>
 #include <stdbool.h>
 #include "swupdate_status.h"
 
-/* Curl Channel Implementation Private Header File.
+/** Curl Channel Implementation Header File.
  *
- * This is a "private" header for testability, i.e., the declarations and
- * definitions herein should be used by code employing the curl channel
- * (e.g. server_hawkbit.c) and unit tests only.
+ * This is the specific channel implementation using libcurl.
  */
 
 typedef enum {
@@ -26,8 +22,13 @@ typedef enum {
 	CHANNEL_POST,
 	CHANNEL_PUT,
 	CHANNEL_PATCH,
+	CHANNEL_DELETE,
 } channel_method_t;
 
+/*
+ * format for a response: the channel can just transfer
+ * or try to parse the answer with a specific format
+ */
 typedef enum {
 	CHANNEL_PARSE_NONE,
 	CHANNEL_PARSE_JSON,
@@ -36,15 +37,20 @@ typedef enum {
 
 #define USE_PROXY_ENV (char *)0x11
 
+/*
+ * Structure to configure the connection and to
+ * exchange data.
+ * This is passed to the channel methods (defined in channel.h)
+ * to set up the connection.
+ */
 typedef struct {
-	char *url;
-	char *cached_file;
+	char *url;		/* URL for connection */
+	char *unix_socket;	/* if set, the UNIX Socket is taken for local connection */
+	char *cached_file;	/* Retrieve file from cached file before getting from network */ 
 	char *auth;
-	char *request_body;
-	char *iface;
-#ifdef CONFIG_JSON
+	char *request_body;	/* Buffer for the answer */
+	char *iface;		/* Set a specific interface */
 	json_object *json_reply;
-#endif
 	char *raw_reply;
 	bool dry_run;
 	char *cafile;
@@ -55,6 +61,7 @@ typedef struct {
 	char *info;
 	char *auth_token;
 	const char *content_type;
+	const char *accept_content_type;
 	unsigned int retry_sleep;
 	unsigned int offs;
 	unsigned int method;
@@ -71,6 +78,11 @@ typedef struct {
 	bool nofollow;
 	size_t (*dwlwrdata)(char *streamdata, size_t size, size_t nmemb,
 				   void *data);
+	/*
+	 * read_fifo is used as alternative to request_body buffer to push data
+	 * in case of large data. This lets push a stream instead of a buffer.
+	 */
+	int read_fifo;
 	size_t (*headers)(char *streamdata, size_t size, size_t nmemb,
 				   void *data);
 	struct swupdate_digest *dgst;
@@ -79,6 +91,7 @@ typedef struct {
 	struct dict *headers_to_send;
 	struct dict *received_headers;
 	unsigned int max_download_speed;
+	size_t	upload_filesize;
 	char *range; /* Range request for get_file in any */
 	void *user;
 } channel_data_t;
