@@ -1,11 +1,10 @@
 /*
- * Copyright (C) 2021 Stefano Babic <sbabic@denx.de>
+ * Copyright (C) 2021 Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
 
-#ifndef _FS_INTERFACE_H
-#define _FS_INTERFACE_H
+#pragma once
 
 char *diskformat_fs_detect(char *device);
 int diskformat_fs_exists(char *device, char *fstype);
@@ -24,4 +23,3 @@ extern int ext_mkfs(const char *device_name, const char *fstype, unsigned long f
 #if defined (CONFIG_BTRFS_FILESYSTEM) 
 extern int btrfs_mkfs(const char *device_name, const char *fstype);
 #endif
-#endif
