@@ -1,12 +1,11 @@
 /*
- * (C) Copyright 2016
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * (C) Copyright 2016-2023
+ * Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
 
-#ifndef _PARSE_LIBRARY_H
-#define _PARSE_LIBRARY_H
+#pragma once
 
 #include <assert.h>
 #include <stdbool.h>
@@ -33,6 +32,7 @@ typedef void (*iterate_callback)(const char *name, const char *value,
 #define config_setting_lookup config_lookup_from
 #endif
 
+bool is_field_numeric_cfg(config_setting_t *e, const char *path);
 void get_value_libconfig(const config_setting_t *e, void *dest);
 void get_field_cfg(config_setting_t *e, const char *path, void *dest);
 void *get_child_libconfig(void *e, const char *name);
@@ -53,11 +53,12 @@ void *get_node_libconfig(config_t *cfg, const char **nodes);
 #define get_field_cfg(e, path, dest)
 #define find_root_libconfig(cfg, nodes, depth)		(NULL)
 #define get_node_libconfig(cfg, nodes)		(NULL)
+#define is_field_numeric_cfg(e, path)	(false)
 #endif
 
-#ifdef CONFIG_JSON
 #include <json-c/json.h>
 
+bool is_field_numeric_json(json_object *e, const char *path);
 const char *get_field_string_json(json_object *e, const char *path);
 void get_value_json(json_object *e, void *dest);
 void get_field_json(json_object *e, const char *path, void *dest);
@@ -72,19 +73,7 @@ char *json_get_data_url(json_object *json_root, const char *key);
 void *find_root_json(json_object *root, const char **nodes, unsigned int depth);
 void *get_node_json(json_object *root, const char **nodes);
 
-#else
-#define find_node_json(a, b, c)		(NULL)
-#define get_field_string_json(e, path)  (NULL)
-#define get_child_json(e, name)		(NULL)
-#define iterate_field_json(e, cb, data)	{ }
-#define get_field_json(e, path, dest)
-#define json_object_object_get_ex(a,b,c) (0)
-#define json_object_array_get_idx(a, b)	(0)
-#define json_object_array_length(a)	(0)
-#define find_root_json(root, nodes, depth)	(NULL)
-#define get_node_json(root, nodes)	(NULL)
-#endif
-
+bool is_field_numeric(parsertype p, void *e, const char *path);
 const char *get_field_string(parsertype p, void *e, const char *path);
 void get_field_string_with_size(parsertype p, void *e, const char *path,
 				char *d, size_t n);
@@ -107,6 +96,3 @@ bool set_find_path(const char **nodes, const char *newpath, char **tmp);
 	d[0] = '\0'; \
 	GET_FIELD_STRING(p, e, name, d); \
 } while (0)
-
-
-#endif
