@@ -1,15 +1,15 @@
 /*
- * (C) Copyright 2016
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * (C) Copyright 2016-2023
+ * Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
 
-#ifndef _SWUPDATE_PCTL_H
-#define _SWUPDATE_PCTL_H
+#pragma once
 
 #include <swupdate_status.h>
 #include <sys/types.h>
+#include "util.h"
 
 extern int pid;
 extern int sw_sockfd;
@@ -31,6 +31,7 @@ void thread_ready(void);
 void wait_threads_ready(void);
 
 typedef int (*swupdate_process)(const char *cfgname, int argc, char **argv);
+typedef server_op_res_t(*server_ipc_fn)(int fd);
 
 void start_subprocess(sourcetype type, const char *name,
 			uid_t run_as_userid, gid_t run_as_groupid,
@@ -43,6 +44,8 @@ void start_subprocess_from_file(sourcetype type, const char *name,
 			const char *cfgfile,
 			int argc, char **argv,
 			const char *cmd);
+/* ipc_thread listens to pctl IPC socket and call server specific function */
+void *ipc_thread_fn(void *data);
 
 void sigchld_handler (int __attribute__ ((__unused__)) signum);
 
@@ -50,5 +53,3 @@ int pctl_getfd_from_type(sourcetype s);
 const char *pctl_getname_from_type(sourcetype s);
 int run_system_cmd(const char *cmd);
 int run_function_background(void *fn, int argc, char **argv);
-
-#endif
