@@ -1,12 +1,11 @@
 /*
- * (C) Copyright 2012-2014
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * (C) Copyright 2013-2023
+ * Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
 
-#ifndef _SWUPDATE_H
-#define _SWUPDATE_H
+#pragma once
 
 #include <sys/types.h>
 #include <stdbool.h>
@@ -15,6 +14,7 @@
 #include "mongoose_interface.h"
 #include "swupdate_dict.h"
 #include "swupdate_image.h"
+#include "hw-compatibility.h"
 
 /*
  * this is used to indicate if a file
@@ -27,14 +27,6 @@ typedef enum {
 	INSTALL_FROM_STREAM
 } swupdate_file_t;
 
-struct hw_type {
-	char boardname[SWUPDATE_GENERAL_STRING_SIZE];
-	char revision[SWUPDATE_GENERAL_STRING_SIZE];
-	LIST_ENTRY(hw_type) next;
-};
-
-LIST_HEAD(hwlist, hw_type);
-
 struct extproc {
 	char name[SWUPDATE_GENERAL_STRING_SIZE];
 	char exec[SWUPDATE_GENERAL_STRING_SIZE];
@@ -72,11 +64,13 @@ struct swupdate_cfg {
 	char current_version[SWUPDATE_GENERAL_STRING_SIZE];
 	char mtdblacklist[SWUPDATE_GENERAL_STRING_SIZE];
 	char forced_signer_name[SWUPDATE_GENERAL_STRING_SIZE];
+	char namespace_for_vars[SWUPDATE_GENERAL_STRING_SIZE];
 	bool syslog_enabled;
 	bool no_downgrading;
 	bool no_reinstalling;
 	bool no_transaction_marker;
 	bool no_state_marker;
+	bool reboot_required;
 	bool check_max_version;
 	int verbose;
 	int loglevel;
@@ -88,11 +82,14 @@ struct swupdate_cfg {
 	struct imglist scripts;
 	struct imglist bootscripts;
 	struct dict bootloader;
+	struct dict vars;
 	struct dict accepted_set;
 	struct proclist extprocs;
 	void *dgst;	/* Structure for signed images */
 	struct swupdate_parms parms;
 	const char *embscript;
+	char gpg_home_directory[SWUPDATE_GENERAL_STRING_SIZE];
+	char gpgme_protocol[SWUPDATE_GENERAL_STRING_SIZE];
 };
 
 #define SEARCH_FILE(img, list, found, offs) do { \
@@ -114,5 +111,3 @@ struct swupdate_cfg {
 int cpio_scan(int fd, struct swupdate_cfg *cfg, off_t start);
 struct swupdate_cfg *get_swupdate_cfg(void);
 void free_image(struct img_type *img);
-
-#endif
