@@ -3,7 +3,7 @@
  * Stefan Herbrechtsmeier <stefan.herbrechtsmeier@weidmueller.com>
  *
  * (C) Copyright 2013
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * Stefano Babic, stefano.babic@swupdate.org.
  *
  * Copyright (c) 2004-2013 Sergey Lyubka
  *
@@ -360,13 +360,24 @@ static void restart_handler(struct mg_connection *nc, void *ev_data)
 static void broadcast_callback(struct mg_connection *nc, int ev,
 		void __attribute__ ((__unused__)) *ev_data, void __attribute__ ((__unused__)) *fn_data)
 {
+	static uint64_t last_io_time = 0;
 	if (ev == MG_EV_READ) {
 		struct mg_connection *t;
 		for (t = nc->mgr->conns; t != NULL; t = t->next) {
-			if (t->data[0] != 'W') continue;
+			if (!t->is_websocket) continue;
 			mg_ws_send(t,(char *)nc->recv.buf, nc->recv.len, WEBSOCKET_OP_TEXT);
 		}
 		mg_iobuf_del(&nc->recv, 0, nc->recv.len);
+		last_io_time = mg_millis();
+	} else if (ev == MG_EV_POLL) {
+		struct mg_connection *t;
+		uint64_t now = *((uint64_t *)ev_data);
+		if (now < last_io_time + 20000) return;
+		for (t = nc->mgr->conns; t != NULL; t = t->next) {
+			if (!t->is_websocket) continue;
+			mg_ws_send(t, "", 0, WEBSOCKET_OP_PING);
+		}
+		last_io_time = now;
 	}
 }
 
@@ -584,7 +595,7 @@ static void upload_handler(struct mg_connection *nc, int ev, void *ev_data,
 				break;
 			}
 
-			swupdate_download_update(0, mp->len, SOURCE_WEBSERVER);
+			swupdate_download_update(0, mp->len);
 
 			if (swupdate_file_setnonblock(fus->fd, true)) {
 				WARN("IPC cannot be set in non-blocking, fallback to block mode");
@@ -637,7 +648,7 @@ static void upload_handler(struct mg_connection *nc, int ev, void *ev_data,
 			percent = (uint8_t)(100.0 * ((double)fus->len / (double)mp->len));
 			if (percent != fus->percent) {
 				fus->percent = percent;
-				swupdate_download_update(fus->percent, mp->len, SOURCE_WEBSERVER);
+				swupdate_download_update(fus->percent, mp->len);
 			}
 
 			fus->last_io_time = mg_millis();
@@ -671,7 +682,6 @@ static void websocket_handler(struct mg_connection *nc, void *ev_data)
 {
 	struct mg_http_message *hm = (struct mg_http_message *) ev_data;
 	mg_ws_upgrade(nc, hm, NULL);
-	nc->data[0] = 'W';
 }
 
 static void ev_handler(struct mg_connection *nc, int ev, void *ev_data, void *fn_data)
