@@ -1,6 +1,6 @@
 /*
  * (C) Copyright 2015-2016
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * Stefano Babic, stefano.babic@swupdate.org.
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
@@ -220,7 +220,7 @@ static bool get_common_fields(parsertype p, void *cfg, struct swupdate_cfg *swcf
 		get_field(p, setting, NULL, &swcfg->reboot_required);
 	}
 
-	TRACE("rebot_required %d", swcfg->reboot_required);
+	TRACE("reboot_required %d", swcfg->reboot_required);
 
 	/*
 	 * Check if SWU should be cached
