@@ -1,6 +1,6 @@
 /*
  * (C) Copyright 2021
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * Stefano Babic, stefano.babic@swupdate.org.
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
@@ -24,7 +24,6 @@
 #include <sys/un.h>
 #include <sys/select.h>
 #include <sys/reboot.h>
-#include <linux/reboot.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <ifaddrs.h>
@@ -647,7 +646,7 @@ static int sysrestart(cmd_t  __attribute__((__unused__)) *cmd, int argc, char *a
 			restart_system(ndevs);
 			sleep(5);
 			sync();
-			if (reboot(LINUX_REBOOT_CMD_RESTART) < 0) { /* It should never happen */
+			if (reboot(RB_AUTOBOOT) < 0) { /* It should never happen */
 				fprintf(stdout, "Please reset the board.\n");
 			}
 			break;
